<?php

namespace qfq;

//use qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../database/Database.php');
require_once(__DIR__ . '/../store/Store.php');


class Sendmail {

    /**
     * @var Store
     */
    private $store = null;

    /**
     * Sends a mail as specified in $mailarr.
     * If there is no receiver specified as 'TO': no mail is sent. This is ok and no error.
     * Logs every send mail as a record in table `mailLog`. Additionally a `grId` and a `xId` can be specified
     * to assign the log entry to a specific action.
     * The log record also contains some information which generates the mail (form/formelement or QFQ query).
     *
     * Structure mailarr:
     *    SENDMAIL_IDX_RECEIVER             email address(es)
     *    SENDMAIL_IDX_SENDER               email address
     *    SENDMAIL_IDX_SUBJECT              string
     *    SENDMAIL_IDX_BODY                 string
     *    SENDMAIL_IDX_REPLY_TO             optional: email address
     *    SENDMAIL_IDX_FLAG_AUTO_SUBMIT     optional: 'on'|'off', default: 'on'
     *    SENDMAIL_IDX_GR_ID                optional: integer
     *    SENDMAIL_IDX_X_ID                 optional: integer
     *
     * @param $mailConfig
     *
     * @throws UserFormException
     */
    public function __construct(array $mailConfig) {

        // If there is no 'Receiver': do not send a mail.
        if (!isset($mailConfig[SENDMAIL_IDX_RECEIVER]) || $mailConfig[SENDMAIL_IDX_RECEIVER] === '') {
            return;
        }

        if (count($mailConfig) < 4 || $mailConfig[SENDMAIL_IDX_SENDER] === '' || $mailConfig[SENDMAIL_IDX_SUBJECT] === '' || $mailConfig[SENDMAIL_IDX_BODY] === '') {
            throw new UserFormException("Error sendmail missing one of: receiver, sender, subject or body", ERROR_SENDMAIL_MISSING_VALUE);
        }

        $this->store = Store::getInstance('');

        $redirectAllMail = $this->store->getVar(SYSTEM_REDIRECT_ALL_MAIL_TO, STORE_SYSTEM);

        if ($redirectAllMail !== false) {
            $addBody = "All QFQ outgoing mails are catched and redirected to you." . PHP_EOL . "Original receiver:" . PHP_EOL;
            $addBody .= 'TO: ' . $mailConfig[SENDMAIL_IDX_RECEIVER] . PHP_EOL;
            $addBody .= 'CC: ' . $mailConfig[SENDMAIL_IDX_RECEIVER_CC] . PHP_EOL;
            $addBody .= 'BCC: ' . $mailConfig[SENDMAIL_IDX_RECEIVER_BCC] . PHP_EOL;
            $addBody .= PHP_EOL . "==========================================" . PHP_EOL . PHP_EOL;

            $mailConfig[SENDMAIL_IDX_BODY] = $addBody . $mailConfig[SENDMAIL_IDX_BODY];

            $mailConfig[SENDMAIL_IDX_RECEIVER] = $redirectAllMail;
            $mailConfig[SENDMAIL_IDX_RECEIVER_CC] = '';
            $mailConfig[SENDMAIL_IDX_RECEIVER_BCC] = '';
        }

        if (empty($mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT]) || $mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT] === '') {
            $mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT] = 'on';
        }

//        $header = $this->buildHeader($mailarr);
//        if (!(mb_send_mail($mailarr[SENDMAIL_IDX_RECEIVER], $mailarr[SENDMAIL_IDX_SUBJECT], $addBody . $mailarr[SENDMAIL_IDX_BODY], $header, "-f " . $mailarr[SENDMAIL_IDX_SENDER]))) {
//            throw new UserFormException("Error sendmail failed.", ERROR_SENDMAIL);
//        }

        $this->sendEmail($mailConfig);
        $this->mailLog($mailConfig);
    }

    /**
     * Use the programm 'sendEmail' - http://caspian.dotconf.net/menu/Software/SendEmail
     *
     * @param array $mailConfig
     * @throws CodeException
     * @throws UserFormException
     */
    private function sendEmail(array $mailConfig) {
        $args = array();

        foreach ($mailConfig as $key => $value) {
            $mailConfig[$key] = Support::escapeDoubleTick($value);
        }

        $args[] = '-f "' . $mailConfig[SENDMAIL_IDX_SENDER] . '"';
        $args[] = '-t "' . $mailConfig[SENDMAIL_IDX_RECEIVER] . '"';
        $args[] = '-o message-charset="utf-8"';
        $args[] = '-q ';

        $logFile = $this->store->getVar(SYSTEM_MAIL_LOG, STORE_SYSTEM);
        if ($logFile != '' && $logFile !== false) {
            $args[] = '-l "' . $logFile . '"';;
        }

        if (!empty($mailConfig[SENDMAIL_IDX_RECEIVER_CC])) {
            $args[] = '-cc "' . $mailConfig[SENDMAIL_IDX_RECEIVER_CC] . '"';;
        }

        if (!empty($mailConfig[SENDMAIL_IDX_RECEIVER_BCC])) {
            $args[] = '-bcc "' . $mailConfig[SENDMAIL_IDX_RECEIVER_BCC];
        }

        if (!empty($mailConfig[SENDMAIL_IDX_SUBJECT])) {
            // The subject needs to be encoded to UTF-8 separately - https://stackoverflow.com/questions/4389676/email-from-php-has-broken-subject-header-encoding/27648245#27648245
            $preferences = ["scheme" => "Q", "input-charset" => "UTF-8", "output-charset" => "UTF-8"];
            $encodedSubject = iconv_mime_encode("Subject", $mailConfig[SENDMAIL_IDX_SUBJECT], $preferences);
            $encodedSubject = substr($encodedSubject, 9); // remove 'Subject: '

            $args[] = '-u "' . $encodedSubject . '"';;
        }

        if (!empty($mailConfig[SENDMAIL_IDX_BODY])) {
            $args[] = '-m "' . $mailConfig[SENDMAIL_IDX_BODY] . '"';;
        }

        if (!empty($mailConfig[SENDMAIL_IDX_REPLY_TO])) {
            $args[] = '-o reply-to="' . $mailConfig[SENDMAIL_IDX_REPLY_TO] . '"';;
        }

        if ($mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT] === 'on') {
            $args[] = '-o message-header="Auto-Submitted: auto-send"';
        }

        if (!empty($mailConfig[SENDMAIL_IDX_ATTACHMENT])) {
            $pieces = explode(',', $mailConfig[SENDMAIL_IDX_ATTACHMENT]);
            foreach ($pieces as $piece) {
                $args[] = '-a ' . $piece;
            }
        }

        if (!empty($mailConfig[SENDMAIL_IDX_HEADER])) {
            $args[] = '-o message-header="' . $mailConfig[SENDMAIL_IDX_HEADER] . '"';
        }

        $sendEmail = $this->store->getVar(SYSTEM_SEND_E_MAIL, STORE_SYSTEM);
        if (empty($sendEmail)) {
            throw new UserFormException("Missing 'sendEmail'", ERROR_SENDMAIL);
        }

        $sendEmailOptions = $this->store->getVar(SYSTEM_SEND_E_MAIL_OPTIONS, STORE_SYSTEM);
        if (!empty($sendEmailOptions)) {
            $args[] = $sendEmailOptions;
        }

        $cmd = $sendEmail . ' ' . implode(' ', $args);

        exec($cmd, $arr, $rc);
        if ($rc != 0) {
            // After first installation of QFQ extension, the PERL script is not executable: is this the case?
            $perms = fileperms($sendEmail);
            if (!($perms & 0x0040)) {
                chmod($sendEmail, 0755);
                exec($cmd, $arr, $rc); // Give it a second try.
            }

            if ($rc != 0) {
                $output = $rc . " - " . implode('<br>', $arr);
                throw new UserFormException("Error sendmail failed: " . $output, ERROR_SENDMAIL);
            }
        }
    }

    /**
     * @param array $mailConfig
     *
     * @return string
     */
    private function buildHeader(array $mailConfig) {

        // "\r\n" needs to be enclosed in double ticks to correctly converted to 0x0d 0x0a,
        $header = "From: " . $mailConfig[SENDMAIL_IDX_SENDER] . "\r\n";

        if (isset($mailConfig[SENDMAIL_IDX_REPLY_TO]) && $mailConfig[SENDMAIL_IDX_REPLY_TO] != '') {
            $header .= "Reply-To: " . $mailConfig[SENDMAIL_IDX_REPLY_TO] . "\r\n";
        }

        // By default 'on'
        if (!isset($mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT]) || $mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT] === 'on') {
            $header .= "Auto-Submitted: auto-send\r\n";
        }

        if (isset($mailConfig[SENDMAIL_IDX_RECEIVER_CC]) && $mailConfig[SENDMAIL_IDX_RECEIVER_CC] != '') {
            $header .= "Cc: " . $mailConfig[SENDMAIL_IDX_RECEIVER_CC] . "\r\n";
        }

        if (isset($mailConfig[SENDMAIL_IDX_RECEIVER_BCC]) && $mailConfig[SENDMAIL_IDX_RECEIVER_BCC] != '') {
            $header .= "Bcc: " . $mailConfig[SENDMAIL_IDX_RECEIVER_BCC] . "\r\n";
        }

        return $header;
    }

    /**
     * Creates a new MailLog Record based on $mailArr / $header.
     *
     * @param array $mailConfig
     *
     * @throws CodeException
     * @throws DbException
     */
    private function mailLog(array $mailConfig) {

        $log = array();

        $header = 'OoO:' . $mailConfig[SENDMAIL_IDX_FLAG_AUTO_SUBMIT];
        if (!empty($mailConfig[SENDMAIL_IDX_HEADER])) {
            $header .= PHP_EOL . 'Custom: ' . $mailConfig[SENDMAIL_IDX_HEADER];
        }
        if (!empty($mailConfig[SENDMAIL_IDX_ATTACHMENT])) {
            $header .= PHP_EOL . 'Attachment: ' . $mailConfig[SENDMAIL_IDX_ATTACHMENT];
        }

        // Log
        $log[] = $mailConfig[SENDMAIL_IDX_RECEIVER];
        $log[] = $mailConfig[SENDMAIL_IDX_SENDER];
        $log[] = $mailConfig[SENDMAIL_IDX_SUBJECT];
        $log[] = $mailConfig[SENDMAIL_IDX_BODY];
        $log[] = $header;
        $log[] = empty($mailConfig[SENDMAIL_IDX_GR_ID]) ? 0 : $mailConfig[SENDMAIL_IDX_GR_ID];
        $log[] = empty($mailConfig[SENDMAIL_IDX_X_ID]) ? 0 : $mailConfig[SENDMAIL_IDX_X_ID];
        $log[] = empty($mailConfig[SENDMAIL_IDX_X_ID2]) ? 0 : $mailConfig[SENDMAIL_IDX_X_ID2];
        $log[] = empty($mailConfig[SENDMAIL_IDX_X_ID3]) ? 0 : $mailConfig[SENDMAIL_IDX_X_ID3];
        $log[] = empty($mailConfig[SENDMAIL_IDX_SRC]) ? 0 : $mailConfig[SENDMAIL_IDX_SRC];

        $db = new Database();
        $db->sql('INSERT INTO MailLog (`receiver`, `sender`, `subject`, `body`, `header`, `grId`, `xId`, `xId2`, `xId3`, `src`, `modified`, `created`) VALUES ( ?, ? , ? , ? ,?, ?, ? ,?, ?, ?, NOW(), NOW() )', ROW_REGULAR, $log);

    }
}
